#ifndef __RESOURCE_nepomuk_ontology_H__
#define __RESOURCE_nepomuk_ontology_H__

#include <gio/gio.h>

G_GNUC_INTERNAL GResource *nepomuk_ontology_get_resource (void);

G_GNUC_INTERNAL void nepomuk_ontology_register_resource (void);
G_GNUC_INTERNAL void nepomuk_ontology_unregister_resource (void);

#endif
