
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "tracker-db-interface-sqlite.h" */

GType tracker_db_interface_flags_get_type (void);
#define TRACKER_TYPE_DB_INTERFACE_FLAGS (tracker_db_interface_flags_get_type())

GType tracker_property_op_get_type (void);
#define TRACKER_TYPE_PROPERTY_OP (tracker_property_op_get_type())

G_END_DECLS

/* Generated data ends here */

