
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "tracker-data-enum-types.h"
#include "tracker-db-interface-sqlite.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "tracker-db-interface-sqlite.h" */

GType
tracker_db_interface_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (TRACKER_DB_INTERFACE_READONLY), "TRACKER_DB_INTERFACE_READONLY", "readonly" },
        { C_FLAGS (TRACKER_DB_INTERFACE_IN_MEMORY), "TRACKER_DB_INTERFACE_IN_MEMORY", "in-memory" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("TrackerDBInterfaceFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
tracker_property_op_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_OP_INSERT), "TRACKER_OP_INSERT", "insert" },
        { C_ENUM (TRACKER_OP_INSERT_FAILABLE), "TRACKER_OP_INSERT_FAILABLE", "insert-failable" },
        { C_ENUM (TRACKER_OP_DELETE), "TRACKER_OP_DELETE", "delete" },
        { C_ENUM (TRACKER_OP_RESET), "TRACKER_OP_RESET", "reset" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerPropertyOp"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

