
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "tracker-sparql-enum-types.h"
#include "tracker-notifier.h"
#include "tracker-connection.h"
#include "tracker-enums.h"
#include "tracker-error.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "tracker-connection.h" */

GType
tracker_sparql_connection_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_NONE), "TRACKER_SPARQL_CONNECTION_FLAGS_NONE", "none" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_READONLY), "TRACKER_SPARQL_CONNECTION_FLAGS_READONLY", "readonly" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_STEMMER), "TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_STEMMER", "fts-enable-stemmer" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_UNACCENT), "TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_UNACCENT", "fts-enable-unaccent" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_STOP_WORDS), "TRACKER_SPARQL_CONNECTION_FLAGS_FTS_ENABLE_STOP_WORDS", "fts-enable-stop-words" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_FTS_IGNORE_NUMBERS), "TRACKER_SPARQL_CONNECTION_FLAGS_FTS_IGNORE_NUMBERS", "fts-ignore-numbers" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_ANONYMOUS_BNODES), "TRACKER_SPARQL_CONNECTION_FLAGS_ANONYMOUS_BNODES", "anonymous-bnodes" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_DISABLE_SYNTAX_EXTENSIONS), "TRACKER_SPARQL_CONNECTION_FLAGS_DISABLE_SYNTAX_EXTENSIONS", "disable-syntax-extensions" },
        { C_FLAGS (TRACKER_SPARQL_CONNECTION_FLAGS_SPARQL_STRICT), "TRACKER_SPARQL_CONNECTION_FLAGS_SPARQL_STRICT", "sparql-strict" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("TrackerSparqlConnectionFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
tracker_serialize_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_SERIALIZE_FLAGS_NONE), "TRACKER_SERIALIZE_FLAGS_NONE", "none" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerSerializeFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
tracker_deserialize_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_DESERIALIZE_FLAGS_NONE), "TRACKER_DESERIALIZE_FLAGS_NONE", "none" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerDeserializeFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "tracker-enums.h" */

GType
tracker_rdf_format_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_RDF_FORMAT_TURTLE), "TRACKER_RDF_FORMAT_TURTLE", "turtle" },
        { C_ENUM (TRACKER_RDF_FORMAT_TRIG), "TRACKER_RDF_FORMAT_TRIG", "trig" },
        { C_ENUM (TRACKER_RDF_FORMAT_JSON_LD), "TRACKER_RDF_FORMAT_JSON_LD", "json-ld" },
        { C_ENUM (TRACKER_RDF_FORMAT_LAST), "TRACKER_RDF_FORMAT_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerRdfFormat"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "tracker-error.h" */

GType
tracker_sparql_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_SPARQL_ERROR_CONSTRAINT), "TRACKER_SPARQL_ERROR_CONSTRAINT", "constraint" },
        { C_ENUM (TRACKER_SPARQL_ERROR_INTERNAL), "TRACKER_SPARQL_ERROR_INTERNAL", "internal" },
        { C_ENUM (TRACKER_SPARQL_ERROR_NO_SPACE), "TRACKER_SPARQL_ERROR_NO_SPACE", "no-space" },
        { C_ENUM (TRACKER_SPARQL_ERROR_ONTOLOGY_NOT_FOUND), "TRACKER_SPARQL_ERROR_ONTOLOGY_NOT_FOUND", "ontology-not-found" },
        { C_ENUM (TRACKER_SPARQL_ERROR_OPEN_ERROR), "TRACKER_SPARQL_ERROR_OPEN_ERROR", "open-error" },
        { C_ENUM (TRACKER_SPARQL_ERROR_PARSE), "TRACKER_SPARQL_ERROR_PARSE", "parse" },
        { C_ENUM (TRACKER_SPARQL_ERROR_QUERY_FAILED), "TRACKER_SPARQL_ERROR_QUERY_FAILED", "query-failed" },
        { C_ENUM (TRACKER_SPARQL_ERROR_TYPE), "TRACKER_SPARQL_ERROR_TYPE", "type" },
        { C_ENUM (TRACKER_SPARQL_ERROR_UNKNOWN_CLASS), "TRACKER_SPARQL_ERROR_UNKNOWN_CLASS", "unknown-class" },
        { C_ENUM (TRACKER_SPARQL_ERROR_UNKNOWN_GRAPH), "TRACKER_SPARQL_ERROR_UNKNOWN_GRAPH", "unknown-graph" },
        { C_ENUM (TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY), "TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY", "unknown-property" },
        { C_ENUM (TRACKER_SPARQL_ERROR_UNSUPPORTED), "TRACKER_SPARQL_ERROR_UNSUPPORTED", "unsupported" },
        { C_ENUM (TRACKER_SPARQL_ERROR_MISSING_LAST_MODIFIED_HEADER), "TRACKER_SPARQL_ERROR_MISSING_LAST_MODIFIED_HEADER", "missing-last-modified-header" },
        { C_ENUM (TRACKER_SPARQL_ERROR_INCOMPLETE_PROPERTY_DEFINITION), "TRACKER_SPARQL_ERROR_INCOMPLETE_PROPERTY_DEFINITION", "incomplete-property-definition" },
        { C_ENUM (TRACKER_SPARQL_ERROR_CORRUPT), "TRACKER_SPARQL_ERROR_CORRUPT", "corrupt" },
        { C_ENUM (TRACKER_SPARQL_ERROR_LAST), "TRACKER_SPARQL_ERROR_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerSparqlError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "tracker-notifier.h" */

GType
tracker_notifier_event_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_NOTIFIER_EVENT_CREATE), "TRACKER_NOTIFIER_EVENT_CREATE", "create" },
        { C_ENUM (TRACKER_NOTIFIER_EVENT_DELETE), "TRACKER_NOTIFIER_EVENT_DELETE", "delete" },
        { C_ENUM (TRACKER_NOTIFIER_EVENT_UPDATE), "TRACKER_NOTIFIER_EVENT_UPDATE", "update" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerNotifierEventType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

